-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
-- Script is the main part of the mission. You can't generally create a mission without some Lua
-- so get ready to some light to medium programming and add http://lua.org , Cortex Command Wiki
-- and forum search to the favourites in your browser.
function MissionTutorial_03:StartActivity()
	print("START! -- MissionTutorial_03:StartActivity()!");
	
	-- Get zone where Dummy dropships will land
	self.DummyLZ = SceneMan.Scene:GetArea("DummyLZ");

	-- Get area with unique name to prevent CC engine from associating your activity with any other
	-- scenes except those for wich you designed your script. CC simply looks at areas defined in 
	-- scene and compares them to areas got in script with SceneMan.Scene:GetArea(). If everything
	-- matches then CC thinks that scene can be used with acitivty script and show scene on map for
	-- selected activity	
	self.Zone = SceneMan.Scene:GetArea("MissionTut_03_Area");
	
	-- Array which stores flags for players that lost their brain.
	self.braindead = {};
	-- Array which stores brains assigned to players. 
	self.AssignedBrains = {};
	-- This is used to limit the amount of ingame movable objects (MO's). Currently CC goes mad if this number exceeds
	-- 255. To avoid unexpected behaviour you MUST always take care of the number of the movable ID's. We'll talk more
	-- about movable objects and limits later when we'll discuss spawning. 
	-- Further reading - http://www.datarealms.com/wiki/index.php/Scenes
	self.MOIDLimit = 200;
    
	-- Here we store the amount of gold that player will recieve at start. Note that we not actually give player
	-- real gold here
	self.PlayerGold = 0; -- Player starting credits

   	-- Team variables are used just for convenience
   	-- Team 2 is always CPU
   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;

	-- Create timers which will countdown spawn intervals for dummies and coalition
	self.DummySpawnTimer = Timer();
	self.CoalitionSpawnTimer = Timer();
	
	-- Set initial spawn intervals. You can change these in AdvanceTo functions as you wission progresses
	self.DummySpawnInterval = 16000;
	self.CoalitionSpawnInterval = 8000;
	
	-- Create lists with weapon names  which will be used when we'll give spawned actors actual weapons
	-- Mission generally starts with lighter weapons and the advances to heavier one. Always double check
	-- weapon names. Error's here don't crash the game or mission, but silently breaks gameplay causing troops
	-- not to spawn. Note that weapon names are case sensitive , so "Shotgun" is ok, but "shotgun" is fail.
	-- The same with troops and dropship names.
	self.CoalitionLightWeapons = {"Shotgun" , "Sniper Rifle" , "Compact Assault Rifle" , "Flamer"};
	self.CoalitionHeavyWeapons = {"Auto Shotgun" , "Heavy Sniper Rifle" , "Assault Rifle"};
	
    self.DummyLightWeapons = {"Rail Pistol" , "Nailgun" , "Blaster" , "Scouting Rifle"};
    self.DummyHeavyWeapons = {"Scouting Rifle" , "Repeater" , "Nailer Cannon" , "Destroyer Cannon" , "Annihiliator"};	
	
   	-- Set up players
	for player = 0, self.PlayerCount - 1 do
        -- Check if we already have a brain assigned
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			-- Create brain here and alter all his neccessary properties
			local brain = CreateAHuman("Brain Robot" , "base.rte");
			brain.Pos = Vector(150 + player * 30 , 550);
			brain:AddInventoryItem(CreateHDFirearm("Auto Pistol" , "Coalition.rte"));
			-- Make brain just stay and do nothing. Further reading - http://www.datarealms.com/wiki/index.php/LuaDocs/Actor
			brain.AIMode = Actor.AIMODE_SENTRY;
			-- Add created brain to the array of assigned actors
			self.AssignedBrains[player] = brain;
            -- Let the spawn into the world, passing ownership
            MovableMan:AddActor(brain);
            -- Set the found brain to be the selected actor at start
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            -- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
			-- Select brain as current actor
			self:SwitchToActor(brain, player, self.PlayerTeam);
        end
    end

    -- Set up AI modes for the Actors that have been added to the scene by the scene definition
	-- Note that actors added by editor are not enumerated in "for actor in MovableMan.Actors"
	-- You need to reset AI behaviour for added actors because otherwise added guards will leave
	-- their positions stalkin for brains (AIMODE_BRAINHUNT). To keep the just staying you need 
	-- to set sentry mode for them
    for actor in MovableMan.AddedActors do
		actor.AIMode = Actor.AIMODE_SENTRY;
    end
	
    -- Set the team's funds. Here we actually give a player some gold.
    self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);
	
	-- Reset timers to avoid unexpected behaviour on activity restart
	self.DummySpawnTimer:Reset();
	self.CoalitionSpawnTimer:Reset();
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function MissionTutorial_03:PauseActivity(pause)
    print("PAUSE! -- MissionTutorial_03:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function MissionTutorial_03:EndActivity()
    print("END! -- MissionTutorial_03:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function MissionTutorial_03:UpdateActivity()
   	-- Clear all objective markers, they get re-added each frame
   	self:ClearObjectivePoints();

	-- We don't have any phases here, just a sandbox combat so run single combat phase
	self:PhaseNoFight();
	
	-- Check for defeat conditions
	self:DoBrainSelection();
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
-- Here we check if we stil have a brain to cobtrol our troops. If not - game over
function MissionTutorial_03:DoBrainSelection()
	-- Check if mission is still running to prevent mission failure
	-- if you waste brain after victory or lose it's object after evacuation
	if not (self.ActivityState == Activity.OVER) then
		-- Iterate through all human players
		for player = 0, self.PlayerCount - 1 do
			-- The current player's team
			local team = self:GetTeamOfPlayer(player);
	
			-- Make sure the game is not already ending
			-- Check if any player's brain is dead
			local brain = self:GetPlayerBrain(player);
			-- Notice the MovableMan:IsActor(brain) function call. This is eesntial when you reference
			-- ingame objects especially actors from lua. It checks if onjects still exists. If you reference 
			-- properties of a non-existent actor (actor can die at any moment) the game will simply crash. 
			-- No fancy console errors at all, just a silent crash.
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				-- Reset player brain
				self:SetPlayerBrain(nil, player);
				-- Try to find a new unasigned brain this player can use instead, or if his old brain entered a craft
				local newBrain = MovableMan:GetUnassignedBrain(team);
				-- Found new brain actor, assign it and keep on truckin'
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					-- All brains of this team are dead. Show fail reason
					FrameMan:SetScreenText("Commander died!", player, 100, -1, false);
					-- Set the flag that this player is brainless
					self.braindead[player] = true;
					
					-- Now see if all brains of self player's team are dead, and if so, end the game
					-- Show Fail\Win banner to participating teams. 
					self.WinnerTeam = self:OtherTeam(team);
					-- Game over
					ActivityMan:EndActivity();
				end
			else
				-- Update the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Spawn dummy dropship
-----------------------------------------------------------------------------------------
function MissionTutorial_03:SpawnDummyDropship()
		-- Here we create desired dropship. Preset names are case-sensitive.
		local ship = CreateACDropShip("Drop Ship", "Dummy.rte");
		-- Actor and weapon are not yet created but we declare them anyway
		local actor = nil;
		local weapon = nil;
		
		-- We add two dummies to the fresh dropship. It's better to do it in loop 
		-- to avoid unnecessary code duplication.
		for i = 0 , 1 do
			-- Creating dummy actor. Preset names are case sensitive.
			actor = CreateAHuman("Dummy" , "Dummy.rte");
			-- Here we select what weapons our actor will hold. This is just a simple random choice 20%
			-- of dummies will get heavy weapons, while others will have light. #self.DummyHeavyWeapons means "number
			-- of elements in self.DummyHeavyWeapons" you should use this instead of magic numbers when working with
			-- tables. math.random(#self.DummyHeavyWeapons) means "random number from 0 to #self.DummyHeavyWeapons".
			-- You can select troops armanent depending on mission phase or whatever you want.
			if math.random() < 0.2 then
				weapon = CreateHDFirearm(self.DummyHeavyWeapons[math.random(#self.DummyHeavyWeapons)] , "Dummy.rte");
			else
				weapon = CreateHDFirearm(self.DummyLightWeapons[math.random(#self.DummyLightWeapons)] , "Dummy.rte");
			end
			-- Add newley created weapon ti dummy inventory
			actor:AddInventoryItem(weapon);
			-- Set dummy team. You should not change team membership on already added actors as it may cause 
			-- unpredicted behevaiour
			actor.Team = self.CPUTeam;
			-- Our dummies will start in brain hunt mode to trying to kill players brain
			actor.AIMode = Actor.AIMODE_BRAINHUNT;

			-- Add actor to drop ship inventory
			ship:AddInventoryItem(actor);
		end -- for

		-- Select random position of dropship inside the specially defined area
		ship.Pos = Vector(self.DummyLZ:GetRandomPoint().X, 0);
		-- Set dropship team
		ship.Team = self.CPUTeam;
		-- Give dropship controls to AI so it could land, unload and return
		ship:SetControllerMode(Controller.CIM_AI, -1);

		-- Add new dropship to the battlefield
		MovableMan:AddActor(ship);
end
-----------------------------------------------------------------------------------------
-- Phase "No fight"
-----------------------------------------------------------------------------------------
function MissionTutorial_03:SpawnClone()
	-- Clones are spawned without dtopships so we cann add them directly to the battle filed. 
	-- Be sure to add troops in non surprising areas. Spawning the troop somewhere in the air
	-- could be a bad idea.
	actor = CreateAHuman("Soldier Light" , "Coalition.rte");
	
	-- Give actor random weapons. Keep in mind that many weapons easily gib after 3 hits, so if you want
	-- to add actor some points of toughness consider giving two weapons, or primary weapon and secondary
	-- 	weapon such as pistols or light machineguns or diggers. AI troopers cant use weapons with fire 
	-- delays, such as miniguns, so don't give them.
	if math.random() < 0.2 then
		weapon = CreateHDFirearm(self.CoalitionHeavyWeapons[math.random(#self.CoalitionHeavyWeapons)] , "Coalition.rte");
	else
		weapon = CreateHDFirearm(self.CoalitionLightWeapons[math.random(#self.CoalitionLightWeapons)] , "Coalition.rte");
	end
	
	actor:AddInventoryItem(weapon);
	actor.Team = self.PlayerTeam;
	-- Clones will be going to the spot were dummies land. You can't generally aim AI trops somewhere on the map 
	-- and expect then to reach destination, so plan troop routes carefully. It's better if troops will move from 
	-- top to bottom. While light dummies can sometimes fly in the sky, clones and their weapons genereally too heavy
	-- to jump over even a small trench. They will simply fall down and gib themselves while trying to climb above.
	actor.AIMode = Actor.AIMODE_GOTO;
	-- Clear all current AI waypoint
	actor:ClearAIWaypoints();
	-- Add new fixed destination
	actor:AddAISceneWaypoint(Vector(1800 , 550));
	-- Set actor position. This actor will apear on the left side of the screen.
	actor.Pos = Vector( 10 , 550);
	-- Add actor to the battlefield
	MovableMan:AddActor(actor);
end
-----------------------------------------------------------------------------------------
-- Phase "No fight"
-----------------------------------------------------------------------------------------
-- This is the initial mission phase. "Historicaly" it is called "no fight".
function MissionTutorial_03:PhaseNoFight()
	-- First we check if enough time left scince last spwan for this team. Then we check if we have enough free
	-- Movable Id's available. You can read about MOID limit here - http://www.datarealms.com/wiki/index.php/Scenes
	-- Keep in mind, that GetMOIDCount does not count MOIDs of actor currently in dropships and rockets, so if you spawn 
	-- many dropships with droops while you have enough free MOIDs, you may break the limit once troops are unloaded.
	-- Breaking the MOID limit is a very bad idea, or player it could be even worse than a simple crash. The game keeps
	-- running without any problems, then it throws some minor glitches, like allies shooting each other or 
	-- bullet holes flying in the air, but that's ok. Then suddenly player's brain just explodes and FAIL starts dancing.
	-- Someone can easily break keyboard seeing this. Depending on battle intensity and amount of dropships I would 
	-- recommend setting self.MOIDLimit to 220-230.
	if self.DummySpawnTimer:IsPastSimMS(self.DummySpawnInterval) and MovableMan:GetMOIDCount() <= self.MOIDLimit then
		self:SpawnDummyDropship();
		-- Reset spawn timer only if we spawned something
		self.DummySpawnTimer:Reset();
	end

	-- The same for coalition
	if self.CoalitionSpawnTimer:IsPastSimMS(self.CoalitionSpawnInterval) and MovableMan:GetMOIDCount() <= self.MOIDLimit then
		self:SpawnClone();
		self.CoalitionSpawnTimer:Reset();
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































